@@ -821,7 +821,7 @@ svc_start(void)
 		svc_start_real();
 }
 
-static void
+static int
 svc_stop_check(RC_SERVICE *state)
 {
 	*state = rc_service_state(service);
@@ -848,7 +848,7 @@ svc_stop_check(RC_SERVICE *state)
 
 	if (*state & RC_SERVICE_STOPPED) {
 		ewarn("WARNING: %s is already stopped", applet);
-		exit(EXIT_SUCCESS);
+		return 1;
 	}
 
 	rc_service_mark(service, RC_SERVICE_STOPPING);
@@ -861,6 +861,8 @@ svc_stop_check(RC_SERVICE *state)
 		else if (rc_service_in_runlevel(service, RC_LEVEL_BOOT))
 			ewarn("WARNING: you are stopping a boot service");
 	}
+
+	return 0;
 }
 
 static void
@@ -986,7 +988,7 @@ svc_stop_real(void)
 	rc_plugin_run(RC_HOOK_SERVICE_STOP_OUT, applet);
 }
 
-static void
+static int
 svc_stop(void)
 {
 	RC_SERVICE state;
@@ -995,13 +997,16 @@ svc_stop(void)
 	if (dry_run)
 		einfon("stop:");
 	else
-		svc_stop_check(&state);
+		if (svc_stop_check(&state) == 1)
+			return 1; /* Service has been stopped already */
 	if (deps)
 		svc_stop_deps(state);
 	if (dry_run)
 		printf(" %s\n", applet);
 	else
 		svc_stop_real();
+
+	return 0;
 }
 
 static void
@@ -1100,9 +1105,11 @@ runscript(int argc, char **argv)
 	bool doneone = false;
 	int retval, opt, depoptions = RC_DEP_TRACE;
 	RC_STRING *svc;
-	char *save = NULL;
+	char path[PATH_MAX], lnk[PATH_MAX];
+	char *dir, *save = NULL, *saveLnk = NULL;
 	char pidstr[10];
 	size_t l = 0, ll;
+ 	const char *file;
 	struct stat stbuf;
 
 	/* Show help if insufficient args */
@@ -1119,7 +1126,40 @@ runscript(int argc, char **argv)
 
 	atexit(cleanup);
 
-	service = xstrdup(argv[1]);
+	/* We need to work out the real full path to our service.
+	 * This works fine, provided that we ONLY allow multiplexed services
+	 * to exist in the same directory as the master link.
+	 * Also, the master link as to be a real file in the init dir. */
+	if (!realpath(argv[1], path)) {
+		fprintf(stderr, "realpath: %s\n", strerror(errno));
+		exit(EXIT_FAILURE);
+	}
+	memset(lnk, 0, sizeof(lnk));
+	if (readlink(argv[1], lnk, sizeof(lnk)-1)) {
+		dir = dirname(path);
+		if (strchr(lnk, '/')) {
+			save = xstrdup(dir);
+			saveLnk = xstrdup(lnk);
+			dir = dirname(saveLnk);
+			if (strcmp(dir, save) == 0)
+				file = basename_c(argv[1]);
+			else
+				file = basename_c(lnk);
+			dir = save;
+		} else
+			file = basename_c(argv[1]);
+		ll = strlen(dir) + strlen(file) + 2;
+		service = xmalloc(ll);
+		snprintf(service, ll, "%s/%s", dir, file);
+		if (stat(service, &stbuf) != 0) {
+			free(service);
+			service = xstrdup(lnk);
+		}
+		free(save);
+		free(saveLnk);
+	}
+	if (!service)
+		service = xstrdup(path);
 	applet = basename_c(service);
 
 	if (argc < 3)
@@ -1316,7 +1356,8 @@ runscript(int argc, char **argv)
 				}
 				if (deps && in_background)
 					get_started_services();
-				svc_stop();
+				if (svc_stop() == 1)
+					continue; /* Service has been stopped already */
 				if (deps) {
 					if (!in_background &&
 					    !rc_runlevel_stopping() &&
